/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
 
package vocleditor.commands.adds;


import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.draw2d.Figure;
import org.eclipse.gef.commands.Command;

import vocleditor.editors.VOCLEditorConstants;
import vocleditor.figure.SetClassifierRoleFigure;
import vocleditor.model.AttributeElement;
import vocleditor.model.VOCLModelManager;
import VOCLModel.Attribute;
import VOCLModel.Constraint;
import VOCLModel.NavExpContainer;
import VOCLModel.OpKind;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModel4Factory;
import VOCLModel.VariableDeclaration;



public class SetClassifierPropsCommand extends Command {
	
	// Names for Label and Description of this Command
	private static final String ConnectionCommand_Label 		= "Add Properties";
	private static final String ConnectionCommand_Description 	= "Adds an property to a collection classifier";
	
	// the associated ClassifierRole & ClassifierRoleFigure
	// and the Container-Class
	protected SetClassifierRole source;
	protected SetClassifierRoleFigure classFigure;
	protected NavExpContainer container;

	private Hashtable changes;
	private Hashtable oldState;
	 
	/**
	 * Constructor, simply calls the superclass
	 * with the ConnectionLabel as parameter.
	 */
	public SetClassifierPropsCommand() {
		super(ConnectionCommand_Label);
	}
	
	/**
	 * This method returns true, if a source (CollClassifierRole, 
	 * see class-members) is specified. 
	 */
	public boolean canExecute() {
		if (source == null)
			return false;
		else
			return true;
	}
	
	/**
	 * This method handles the execution of changing properties
	 * of a class (CollClassifierRole). Therefor it uses a NavExpModelFactory
	 * to create Attributes and sets the the corresponding
	 * references to the CollClassifierRole.
	 * Finally the CollClassifierRoleFigure is updated.
	 */
	public void execute() {
		// first retrieve all settings for
		// undo functionality
		oldState = this.getInitialValues();
		
		// now execute the update
		this.executeUpdate(changes);		
	}

	/**
	 * Retrieves the description.
	 */
	public String getLabel() {
		return ConnectionCommand_Label; // Description; // test Label instead
	}

	/**
	 * Retrieves the corresponding ClassifierRole
	 * @return - the ClassifierRole
	 */
	public SetClassifierRole getSource() {
		return source;
	}


	/**
	 * Simply call with stored variables
	 * (oldState, ..)
	 */
	public void undo() {
		this.executeUpdate(oldState);
	}

	/**
	 * Redo this action means 
	 * simply call executeUpdate with
	 * stored variables.
	 */
	public void redo() {
		this.executeUpdate(changes);
	}

	/**
	 * Sets the ClassifierRole for this Action.
	 * @param newSource - the ClassifierRole
	 */
	public void setSource(SetClassifierRole newSource) {
		source = newSource;
	}

	/**
	 * Returns the container-class of this model.
	 * @return - a NavExpContainer
	 */
	public NavExpContainer getContainer() {
		return container;
	}

	/**
	 * Sets the container-class for this model.
	 * @param container
	 */
	public void setContainer(NavExpContainer container) {
		this.container = container;
	}
	
	/**
	 * retrieve the figure form this command
	 * @param figure - the main figure of the CollClassifierRole
	 */
	public Figure getClassFigure() {
		return classFigure;
	}
	

	/**
	 * This method sets the used figure for this action.
	 * Should be the corresponding figure of the ClassifierRole
	 * @param figure 
	 */
	public void setClassFigure(SetClassifierRoleFigure figure) {
		classFigure = figure;
	}
	
	/**
	 * This Method makes the updates in the figure and the model
	 * and is also used for undo functionality.
	 */
	public void executeUpdate(Hashtable changes) {
		int state 			= ((Integer)changes.get(VOCLEditorConstants.STATE)).intValue();
		String countName 	= (String)changes.get(VOCLEditorConstants.SET_SIZE_VAR);
		String globalName 	= (String)changes.get(VOCLEditorConstants.CLASS_VAR_NAME);
		String summandName 	= (String)changes.get(VOCLEditorConstants.SUMMAND_VAR);
		// executes the simple settings (without attributes)
 		executeUpdate(state, countName, summandName, globalName);
		
		// retrieve the container for storing variable-declarations
		VOCLExpression exp = container;
		Constraint con = null;
		while(con==null){
			if(exp.eContainer() instanceof Constraint)con=exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}
		
		// 1. remove all attributes
		Iterator i = source.getAttribute().iterator();
		while(i.hasNext()){
			Attribute att=(Attribute)i.next();
			if(att.getVariableDeclaration()!=null) {
				con.getVariableDeclaration().remove(att.getVariableDeclaration());
			}
		}
		classFigure.removeAllAttributes();
		source.getAttribute().clear();

		// 2. add all attributes 
		VOCLModel4Factory mfc = VOCLModelManager.getFactory();
		Iterator it = ((LinkedList)changes.get(VOCLEditorConstants.ATTRIBUTES)).iterator();
		while(it.hasNext()){
			AttributeElement attElem=(AttributeElement)it.next();
			Attribute att=mfc.createAttribute();
			att.setName(attElem.getAttName());
			att.setType(attElem.getType());
			if(attElem.isVar()){
				VariableDeclaration var=mfc.createVariableDeclaration();
				var.setVarName(attElem.getVarname());
				var.setType(attElem.getType());
				att.setValue("");
				con.getVariableDeclaration().add(var);
				att.setVariableDeclaration(var);
				
				// set the classFigure with varName for attValue
				classFigure.addAttribute(att.getName(),att.getType(), attElem.getVarname(), false);
			}else{
				att.setValue(attElem.getValue());
				// set the classFigure with attValue 
				classFigure.addAttribute(att.getName(),att.getType(), att.getValue(), true);
			}
			
			source.getAttribute().add(att);	
		}
	}
	
	/**
	 * This Method makes the updates in the figure and the model
	 * and is also used for undo functionality.
	 */
	public void executeUpdate(int state, String countName, String summandName, String globalName) {
		
		try {
			//		here check what happend
			classFigure.setSetOperation(state, countName, summandName);
			classFigure.setClassVarName(globalName);
	
			// update the model
			// 1. setting the opkind
			switch(state) {
				case OpKind.COUNT:
					source.setOpKind(OpKind.COUNT_LITERAL);
					// here also set the variable
					VariableDeclaration vdcl = source.getOpVar();
					if (vdcl == null) {
						VOCLModel4Factory mfc = VOCLModelManager.getFactory();
						vdcl = mfc.createVariableDeclaration();
					} 
					vdcl.setType("Integer");
					vdcl.setVarName(countName);
					
					addVarDeclToConstraint(vdcl);
					source.setOpVar(vdcl);
					break;
				case OpKind.EMPTY:
					source.setOpKind(OpKind.EMPTY_LITERAL);
					break;
				case OpKind.NOT_EMPTY:
					source.setOpKind(OpKind.NOT_EMPTY_LITERAL);
					break;
				case OpKind.NO_OP:
					source.setOpKind(OpKind.NO_OP_LITERAL);
					break;
				case OpKind.SUM:
					source.setOpKind(OpKind.SUM_LITERAL);
					
					// set the operation var (the resulting sum)
					vdcl = source.getOpVar();
					if (vdcl == null) {
						VOCLModel4Factory mfc = VOCLModelManager.getFactory();
						vdcl = mfc.createVariableDeclaration();
					} 
					vdcl.setType("Integer");
					vdcl.setVarName(countName);
	
					addVarDeclToConstraint(vdcl);
					source.setOpVar(vdcl);
					
					// set the summand (referring to an attribute)
					vdcl = source.getIteratorVar();
					if (vdcl == null) {
						VOCLModel4Factory mfc = VOCLModelManager.getFactory();
						vdcl = mfc.createVariableDeclaration();
					} 
					vdcl.setType("Integer");
					vdcl.setVarName(summandName);
	
					addVarDeclToConstraint(vdcl);
					source.setIteratorVar(vdcl);
		
					break;
			}
			if(globalName.trim().length()>0){
				if(source.getCallVar()!=null)
					source.getCallVar().setVarName(globalName);
				else{
						VOCLModel4Factory mfc = VOCLModelManager.getFactory();
						VariableDeclaration callVar = mfc.createVariableDeclaration(); 
						callVar.setVarName(globalName);
						callVar.setType(source.getType());
						addVarDeclToConstraint(callVar);
						source.setCallVar(callVar);
				}
			}else{
				if(source.getCallVar()!=null){
						removeVarDeclToConstraint(source.getCallVar());
						source.setCallVar(null);
				}
				
			}
		} catch (Exception e) {
			System.out.println("Error in:");
			e.printStackTrace();
		}

	}
	
	/**
	 * Getter for the Attributes nested in this class
	 * @return - List of all attributes
	 */
	public LinkedList getAttributes() {
		LinkedList attributes = new LinkedList();
		Iterator it=source.getAttribute().iterator();
		while(it.hasNext()){
			Attribute att=(Attribute)it.next();
			AttributeElement attElem=new AttributeElement();
			attElem.setAttName(att.getName());
			attElem.setType(att.getType());
			if(att.getVariableDeclaration()!=null){
				VariableDeclaration var=att.getVariableDeclaration();
				attElem.setVarname(var.getVarName());
				attElem.setValue("");
				attElem.setVar(true);
			}else{
				attElem.setValue(att.getValue());
				attElem.setVar(false);
			}
			attributes.add(attElem);	
		}		
			
		return attributes;
	}
	
	/**
	 * Method to add the given VariableDeclaration
	 * to the Constraint-Object
	 */
	public void addVarDeclToConstraint (VariableDeclaration decl) {
		Constraint con = null;
		VOCLExpression exp=(VOCLExpression)container;
		while(con==null){
			if(exp.eContainer() instanceof Constraint)con=exp.getConstraint();
			else exp=(VOCLExpression) exp.eContainer();	
		}
		con.getVariableDeclaration().add(decl);
	}
	
	
	/**
	 * Method to remove the given VariableDeclaration
	 * to the Constraint-Object (if is inside)
	 */
	public void removeVarDeclToConstraint (VariableDeclaration decl) {
		Constraint con = null;
		VOCLExpression exp=(VOCLExpression)container.eContainer();
		while(con==null){
			if(exp instanceof Constraint)con=(Constraint) exp;
			else exp=(VOCLExpression) exp.eContainer();	
		}
		con.getVariableDeclaration().remove(decl);
	}
	
	/**
	 * Retrieves the changes made during 
	 * the execution of this command
	 * @return changes
	 */
	public Hashtable getChanges() {
		return changes;
	}

	/**
	 * Sets the changes to be made by
	 * this command
	 * @param hashtable - the changes
	 */
	public void setChanges(Hashtable hashtable) {
		changes = hashtable;
	}
	
	/**
	 * Method fills the hashtable 'changes' with all initial Values, by usage 
	 * of AttributeElement objects instead of the attributes themselve
	 * @return hashtable filled with all necessary values of a setClassifierRole,
	 * e.g. classname, classvarname, opKind and attributes
	 **/
	public Hashtable getInitialValues() {
		Hashtable hash = new Hashtable();
		String varname="";
		if(source.getCallVar()!=null)varname=source.getCallVar().getVarName();
		hash.put(VOCLEditorConstants.CLASS_VAR_NAME, varname);
		hash.put(VOCLEditorConstants.CLASS_NAME, source.getType());
		hash.put(VOCLEditorConstants.STATE, new Integer(source.getOpKind().getValue()));	

		String vn 		= "";
		String summand 	= "";
		VariableDeclaration decCount 	= source.getOpVar();
		VariableDeclaration decSummand 	= source.getIteratorVar();
		// if they are set, retrieve the names
		if (decCount != null)
			vn = decCount.getVarName();
		if (decSummand != null) 
			summand = decSummand.getVarName();

		hash.put(VOCLEditorConstants.SET_SIZE_VAR, vn);
		hash.put(VOCLEditorConstants.SUMMAND_VAR, summand);			

		hash.put(VOCLEditorConstants.ATTRIBUTES, this.getAttributes());
	
		return hash;
	}

}