/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;



import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policy.ExpressionComponentEditPolicy;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.VOCLExpression;



/**
 * Class       :ExpressionEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for VOCLExpression
 * <br/>
 * @author vilapower
 */
public abstract class AbstractExpressionEditPart extends AbstractGraphicalEditPart implements Adapter{
	
	private IPropertySource propertySource = null;
	private Notifier target;
	


	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new ExpressionComponentEditPolicy());
	}

	/**
	 * returns model of this EditPart
	 * @return VOCLExpression
	 */
	public abstract VOCLExpression getExpression();

	
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive())return;
		// start listening for changes in the model
		getExpression().eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive())return;
		// stop listening for changes in the model
		getExpression().eAdapters().remove(this);
		super.deactivate();
	}



	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getExpression() );
		}
		return propertySource;
		
	}
	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		switch( type ) {
			case Notification.ADD:
			case Notification.REMOVING_ADAPTER:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
				refreshChildren();
				refreshVisuals();
				refreshChildren();
				refreshVisuals();	
				refreshChildren();
				refreshVisuals();	
				break;
			case Notification.SET:
				refreshChildren();
				refreshVisuals();
				break;
		}		
	}



	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {	
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			BorderLayout.CENTER);
	}
	
	/**
	 * returns Figure,that is the parent for the figure of child
	 * @param child Editpart
	 * @return IFigure
	 */
	public abstract IFigure getPrivateContentPane(EditPart child);
	
	/**
	 * returns Figure,that is the parent for the figure of child
	 * @param child Object as model
	 * @return IFigure
	 */
	public abstract IFigure getPrivateContentPane(Object child) ;
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#addChildVisual(org.eclipse.gef.EditPart, int)
	 */
	protected void addChildVisual(EditPart childEditPart, int index) {
		IFigure child = ((GraphicalEditPart)childEditPart).getFigure();
		getPrivateContentPane(childEditPart).add(child, index);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#removeChildVisual(org.eclipse.gef.EditPart)
	 */
	protected void removeChildVisual(EditPart childEditPart) {
		IFigure child = ((GraphicalEditPart)childEditPart).getFigure();
		getPrivateContentPane(childEditPart).remove(child);

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#reorderChild(org.eclipse.gef.EditPart, int)
	 */
	protected void reorderChild(EditPart child, int index) {
		// Save the constraint of the child so that it does not
		// get lost during the remove and re-add.
		IFigure contentpane=getPrivateContentPane(child);
		IFigure childFigure = ((GraphicalEditPart) child).getFigure();
		LayoutManager layout = contentpane.getLayoutManager();
		Object constraint = null;
		if (layout != null)
			constraint = layout.getConstraint(childFigure);
	
		removeChildVisual(child);
		List children = getChildren();
		children.remove(child);
		children.add(index, child);
		addChildVisual(child, index);
		
		contentpane.setConstraint(childFigure, constraint);
	}

	/**
	 * @see org.eclipse.gef.GraphicalEditPart#setLayoutConstraint(org.eclipse.gef.EditPart, org.eclipse.draw2d.IFigure, java.lang.Object)
	 */
	public void setLayoutConstraint(EditPart child,IFigure childFigure, Object constraint) {
		getPrivateContentPane(child).setConstraint(childFigure, constraint);
	}

	
	/**
	 * refreshed children of this EditPart
	 * 
	 * @param modelObjects - List of model children
	 * @param modelToEditPart -Map where model children are references to theis EditPart
	 * @param children List of editparts children
	 */
	protected void refreshPrivateChildren(List modelObjects,Map modelToEditPart,List children){
		int i;
		Object model;
		EditPart editPart;
		
		for (i = 0; i < modelObjects.size(); i++) {
			model = modelObjects.get(i);
			//Do a quick check to see if editPart[i] == model[i]
			if (i < children.size()
				&& ((EditPart) children.get(i)).getModel() == model)
					continue;
			//Look to see if the EditPart is already around but in the wrong location
			editPart = (EditPart)modelToEditPart.get(model);
			if (editPart != null)
				reorderChild (editPart, i);
			else {
				//An editpart for this model doesn't exist yet.  Create and insert one.
				editPart = createChild(model);
				addChild(editPart, i);
			}
		}
		List trash = new ArrayList();
		for (; i < children.size(); i++)
			trash.add(children.get(i));
		for (i = 0; i < trash.size(); i++) {
			EditPart ep = (EditPart)trash.get(i);
			removeChild(ep);
		}				
	}
	
	/**
	 * Expression can be never selected
	 * @see org.eclipse.gef.EditPart#setSelected(int)
	 */
	public void setSelected(int value) {}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		return null;
	}



}
