/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.figure;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;

import vocleditor.editors.VOCLEditorConstants;
import VOCLModel.Attribute;
import VOCLModel.VariableDeclaration;


public class ClassifierRoleLabel extends Figure{
	
	// the Label for the classifier-head 
	protected Label classLabel;
	// the color used for this figure on background
	
	// this figure consists of two compartment-figures:
	// one for the classifier-head (class- and variable-name)
	// and one for the list of attributes
	private CompartmentFigure attributeFigure = new CompartmentFigure();
	private CompartmentFigure classFigure = new CompartmentFigure();
	private String className;
	private String varName;
	
	/**
	 * Constructor of the ClassifierRoleFigure:
	 * Creates a Figure for a class with a class-name,
	 * a variable-name (optional - may be empty, but not null),
	 * and attributes and place them inside a UML-like
	 * box for classes.
	 * @param shell     - the shell where the figure is drawn on
	 * @param varname   - the variable-name for this class-instance
	 * @param classname - the class name
	 */
	public ClassifierRoleLabel(String varname,String classname){
		
		// store the values of class and variablename
		this.varName   = varname;
		this.className = classname;
			
		// The classlabel with <@img><varname>: <classname> and the font specified above
		classLabel = new Label(varname + ": " + classname,
						 new Image(null, 
		                       ClassifierRoleLabel.class.getResourceAsStream("gif/class_obj.gif")));
		
		classLabel.setFont(VOCLEditorConstants.classFont);
		
		// place everything inside a ToolbarLayout
		// and a LineBorder
		ToolbarLayout layout = new ToolbarLayout();
		setLayoutManager(layout);
		setBorder(new LineBorder(ColorConstants.black,1));
		setBackgroundColor(VOCLEditorConstants.classColor);
		setOpaque(true);
		
		// add the nested components to the figure
		classFigure.add(classLabel);
		add(classFigure);
		add(attributeFigure);
	}



	
	/**
	 * Getter method for the CompartmentFigure
	 * nested in this class for the attributes.
	 * @return - the CompartmentFigure for the attributes.
	 */
	public CompartmentFigure getAttributesCompartment() {
		return attributeFigure;
	}

	/**
	 * A method for specifying the variable- and classname
	 * of this ClassifierRoleFigure.
	 * Changes only the label, but not the values.
	 * @param varname 	- the name of the variable
	 * @param classname	- the name of the class
	 */
	public void setName(String varname,String classname){
		classLabel.setText(varname.concat(": ").concat(classname));	
	}
		
	/**
	 * Overwritten method, indicates that a local
	 * coordinate system is used.
	 */
	protected boolean useLocalCoordinates() {return true;}

	/**
	 * Retrieving the minimum Dimension
	 * @see getPreferredSize()
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {return getPreferredSize();}

	/**
	 * Retrieving the wanted Dimension for this figure.
	 * It is calculated by the sum of the dimension of the
	 * classFigure (at the top) and the dimension for the
	 * attributeFigure (bottom) in y-axis, and the max of
	 * the values from both for x-axis.
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		Dimension dim1 = classFigure.getPreferredSize(0,0);
		Dimension dim2 = attributeFigure.getPreferredSize(0,0);	
		int x = Math.max(dim1.width,dim2.width);
		int y = dim1.height + dim2.height;
		return new Dimension(x, y);
	}
	
	/**
	 * This method adds an Attribute to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param name	- the attributes name
	 * @param type	- the attributes type (e.g. String)
	 * @param value	- the attributes value
	 */
	public void addAttribute(String name, String type, String value, boolean isValue) {
		Image img;
		if (isValue){
			img = new Image(null, 
			ClassifierRoleLabel.class.getResourceAsStream("gif/att.gif"));
		} else {
			img = new Image(null, 
						ClassifierRoleLabel.class.getResourceAsStream("gif/attvar.gif"));
		}
		Label attLabel = new Label(name + ": \"" + value + "\" [" + type + "]", img);
		attributeFigure.add(attLabel);	
	}
	
	
	/**
	 * This method adds  Attributes to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param list	- list of attributes
	 */
	public void setAttributes(List list) {
		attributeFigure.removeAll();
		Iterator it=list.iterator();
		while(it.hasNext()){
			Attribute att=(Attribute)it.next();

			Image img;
			if (!(att.getVariableDeclaration() != null)){
				img = new Image(null, 
				ClassifierRoleLabel.class.getResourceAsStream("gif/att.gif"));
			} else {
				img = new Image(null, 
							ClassifierRoleLabel.class.getResourceAsStream("gif/attvar.gif"));
			}			
			
			String str=att.getName() + ": \"";
			if(att.getVariableDeclaration()!=null){
				VariableDeclaration var=att.getVariableDeclaration();
				str=str+ var.getVarName();
			}else
				str=str+ att.getValue();
			str=str+ "\" [" + att.getType() + "]";
			Label attLabel=new Label(str,img);

			attributeFigure.add(attLabel);	
		}
	}
	
	/**
	 * This method adds an Attribute to the figure, 
	 * specified by name, type and value.
	 * Inserted as Label in the following format:
	 * <name>: "<value>" [<type>]
	 * @param name	- the attributes name
	 * @param type	- the attributes type (e.g. String)
	 * @param value	- the attributes value
	 */
	public void changeAttribute(int index, String name, String type, String value) {
		Label label = (Label)((attributeFigure.getChildren()).get(index));
		label.setText(name + ": \"" + value + "\" [" + type + "]");
	}
	
	/**
	 * This method removes an element from the Attributelist
	 * specified by index.
	 */
	public void removeAttribute(int index) {
		List children = attributeFigure.getChildren();
		Figure f = (Figure)children.get(index);
		attributeFigure.remove(f);
	}
	
	/**
	 * This method will set the ClassVarName of this 
	 * classifier in the graphical view.
	 */
	public void setClassVarName(String name) {
		this.varName = name;
		classLabel.setText(varName + ": " + className);
	}
	
	public void removeAllAttributes() {
		attributeFigure.removeAll();
	}
}
